###########################################
### Script to identify the "best" runs  ###
### based on their objective function,  ###
### AIC, AICc, BIC                      ###
###########################################

files_run <- dir(pattern="data_retro", path=paste(dirRData))

res <- NULL
for (i in 1:length(files_run)){
  load(paste(dirRData, files_run[i], sep=""))
  run_keys  <-  unlist(strsplit(files_run[i], split="[.]"))[1]
  split_run <- unlist(strsplit(run_keys, split="_"))
  ind1      <- length(split_run)
  seed_nb   <- split_run[length(split_run)]
  ind2      <- which(split_run=="Run")
  run_name  <- paste(split_run[ind2:(ind1-1)], collapse="_")
  res <- rbind(res, data.frame(run=run_name,
                               seed_nb     = seed_nb,
                               obj         = outputs_VPA$obj$obj_func,
                               obj_wit_cte = outputs_VPA$obj$obj_func_with_cte,
                               nb_param    = outputs_VPA$obj$nb_param,
                               nb_data     = outputs_VPA$obj$nb_data,
                               AIC         = outputs_VPA$obj$AIC,
                               AICc        = outputs_VPA$obj$AICc,
                               BIC         = outputs_VPA$obj$BIC,
                               chi         = outputs_VPA$obj$chi_square))
  
}

save(res, file=paste(dirRData, "res.Rdata", sep=""))

dir_bests=NULL
for (i in 1:length(unique(res$run))){
  subset_runs <- subset(res, subset=c(res$run==unique(res$run)[i]))
  subset_runs$obj <- as.numeric(as.character(subset_runs$obj))
  subset_runs$seed_nb <- as.numeric(as.character(subset_runs$seed_nb))  
  iwh_min <- subset_runs[which(subset_runs$obj==min(subset_runs$obj)),]
  if (sum(iwh_min$seed_nb==911)==0){dir_run_best = res[as.numeric(rownames(iwh_min)[1]),]} else {
    dir_run_best = res[as.numeric(rownames(iwh_min[which(iwh_min$seed_nb==911),])),]
  }
  dir_bests <- rbind(dir_bests, dir_run_best)
}
dir_bests$seed_nb <- as.numeric(as.character(dir_bests$seed_nb))
save(dir_bests, file= paste(dirRData, "dir_best.Rdata", sep=""))


dir_bests_AIC=NULL
for (i in 1:length(unique(res$run))){
  subset_runs <- subset(res, subset=c(res$run==unique(res$run)[i]))
  subset_runs$obj <- as.numeric(as.character(subset_runs$obj))
  subset_runs$AIC <- as.numeric(as.character(subset_runs$AIC))
  subset_runs$seed_nb <- as.numeric(as.character(subset_runs$seed_nb))  
  iwh_min <- subset_runs[which(subset_runs$AIC <= (min(subset_runs$AIC)+5)),]
  if (sum(iwh_min$seed_nb==911)==0){dir_run_best = res[as.numeric(rownames(iwh_min)[1]),]} else {
    dir_run_best = res[as.numeric(rownames(iwh_min[which(iwh_min$seed_nb==911),])),]
  }
  dir_bests_AIC <- rbind(dir_bests_AIC, dir_run_best)
}
dir_bests_AIC$seed_nb <- as.numeric(as.character(dir_bests_AIC$seed_nb))
save(dir_bests_AIC, file= paste(dirRData, "dir_best_AIC.Rdata", sep=""))

dir_bests_AICc=NULL
for (i in 1:length(unique(res$run))){
  subset_runs <- subset(res, subset=c(res$run==unique(res$run)[i]))
  subset_runs$obj <- as.numeric(as.character(subset_runs$obj))
  subset_runs$AICc <- as.numeric(as.character(subset_runs$AICc))
  subset_runs$seed_nb <- as.numeric(as.character(subset_runs$seed_nb))  
  iwh_min <- subset_runs[which(subset_runs$AICc <= (min(subset_runs$AICc)+5)),]
  if (sum(iwh_min$seed_nb==911)==0){dir_run_best = res[as.numeric(rownames(iwh_min)[1]),]} else {
    dir_run_best = res[as.numeric(rownames(iwh_min[which(iwh_min$seed_nb==911),])),]
  }
  dir_bests_AICc <- rbind(dir_bests_AICc, dir_run_best)
}
dir_bests_AICc$seed_nb <- as.numeric(as.character(dir_bests_AICc$seed_nb))
save(dir_bests_AICc, file= paste(dirRData, "dir_best_AICc.Rdata", sep=""))


dir_bests_BIC=NULL
for (i in 1:length(unique(res$run))){
  subset_runs <- subset(res, subset=c(res$run==unique(res$run)[i]))
  subset_runs$obj <- as.numeric(as.character(subset_runs$obj))
  subset_runs$BIC <- as.numeric(as.character(subset_runs$BIC))
  subset_runs$seed_nb <- as.numeric(as.character(subset_runs$seed_nb))  
  iwh_min <- subset_runs[which(subset_runs$BIC <= (min(subset_runs$BIC)+5)),]
  if (sum(iwh_min$seed_nb==911)==0){dir_run_best = res[as.numeric(rownames(iwh_min)[1]),]} else {
    dir_run_best = res[as.numeric(rownames(iwh_min[which(iwh_min$seed_nb==911),])),]
  }
  dir_bests_BIC <- rbind(dir_bests_BIC, dir_run_best)
}
dir_bests_BIC$seed_nb <- as.numeric(as.character(dir_bests_BIC$seed_nb))
save(dir_bests_BIC, file= paste(dirRData, "dir_best_BIC.Rdata", sep=""))

dir_bests_tot <- rbind(cbind(dir_bests, criteria="Objective"), cbind(dir_bests_AIC, criteria="AIC"), cbind(dir_bests_AICc, criteria="AICc"), cbind(dir_bests, criteria="BIC"))
save(dir_bests_tot, file= paste(dirRData, "dir_bests_tot.Rdata", sep=""))
