##########################################################################
######   Script to make the different plots of the VPA output files   ####
##########################################################################

##############################################################
###  Source the different scripts to read the VPA outputs  ###
###  These scripts should be fine#                         ###
##############################################################
source(paste(dirRScripts, "io.funcs.R", sep=""))
source(paste(dirRScripts, "io.VPA2Box.R", sep=""))
source(paste(dirRScripts, "io.Retros.R", sep=""))
source(paste(dirRScripts, "kobe-2box.R", sep=""))


source(paste(dirRScripts, "OutputsVPA2NetCDF.R", sep=""))

###################################################################
### Here are a set of functions to read and plot the outputs    ###
### get_retros_data :  get the data from a retrospective analysis##
###                    using the  directory of the run (dir_in) ###
###                 directory of the run (dir_in)               ###
###################################################################

get_directories <- function(run=run_to_get, seed_nb=seed_number_to_get, data_type="Retros", dir_run=dir_main, catch_type="Reported"){
  dir_runs    <- list.dirs(paste(dir_run, catch_type, "/", run_to_get, "/", data_type, "/", seed_nb, sep=""))
  dir_runs2   <- as.data.frame(cbind(i=1:length(dir_runs), path=dir_runs))
  dir_runs2$i <- as.numeric(as.character(dir_runs2$i))
  dir_runs3   <- ddply(dir_runs2, .(i), function(x) length(unlist(strsplit(as.character(x$path[1]), split="/"))) )
  iwh         <- which(dir_runs3$V1==10)
  dir_runs    <- dir_runs[iwh]
  iwh2        <- grep(data_type, dir_runs)
  dir_runs    <- dir_runs[iwh2]
}


#################################################################
### Basic function to read the VPA output of a retrospective  ###
### analysis                                                  ###
#################################################################
get_retros_data <- function(dir_in, name_run, age_plus=10, nRet=5){
  outputs_VPA       <- NULL
  dir_run           <- paste(dir_in,'/', sep="")
  m                 <- c(0.4900,0.2400, 0.2400,0.2400,0.2400,0.2000,0.1750,0.1500,0.1250,rep(0.1000,(age_plus-9)))
  run2              <- readVpa2boxFn(paste(dir_run, dir(".c1", path=dir_in)[1], sep=""), m=m)
  plts_tot <- NULL 
  scan_res           <- scan(paste(dir_in,'/MINUS0.R', sep=""), what="", sep="\n", strip.white=TRUE)
  
  outputs_VPA$obj$obj_func          <- unlist(strsplit(scan_res[8], split=" "))[length(unlist(strsplit(scan_res[8], split=" ")))]
  outputs_VPA$obj$obj_func_with_cte <- unlist(strsplit(scan_res[9], split=" "))[length(unlist(strsplit(scan_res[9], split=" ")))]
  outputs_VPA$obj$nb_param          <- unlist(strsplit(scan_res[10], split=" "))[length(unlist(strsplit(scan_res[10], split=" ")))]
  outputs_VPA$obj$nb_data           <- unlist(strsplit(scan_res[11], split=" "))[length(unlist(strsplit(scan_res[11], split=" ")))]
  outputs_VPA$obj$AIC               <- unlist(strsplit(scan_res[12], split=" "))[length(unlist(strsplit(scan_res[12], split=" ")))]
  outputs_VPA$obj$AICc              <- unlist(strsplit(scan_res[13], split=" "))[length(unlist(strsplit(scan_res[13], split=" ")))]
  outputs_VPA$obj$BIC               <- unlist(strsplit(scan_res[14], split=" "))[length(unlist(strsplit(scan_res[14], split=" ")))]
  outputs_VPA$obj$chi_square        <- unlist(strsplit(scan_res[15], split=" "))[length(unlist(strsplit(scan_res[15], split=" ")))]

  cpue_fit <- scan_res
  CPUE_names         <- substr(cpue_fit[grep(pattern = "Discrepancy", cpue_fit)-9],5,max(nchar(cpue_fit[grep(pattern = "Discrepancy", cpue_fit)-9])))
  fit_tot <- NULL
  
  for (i in 1:length(CPUE_names)){
    ind1 <- grep(CPUE_names[i],cpue_fit)+11
    ind2 <- (grep(pattern = "Selectivities by age", cpue_fit)-1)[i]
    if (ind1<=ind2){
    tablefit <- data.frame(k=1:(ind2-ind1+1), values=as.character(cpue_fit[ind1:ind2]))
    tablefit2 <- ddply(tablefit, .(k), 
                       function(x) as.numeric(unlist(strsplit(as.character(x$values),split=" "))[which(nchar(unlist(strsplit(as.character(x$values),split=" ")))>0)]))
    fit_tot <- rbind(fit_tot,data.frame(CPUE=CPUE_names[i], Year=tablefit2$V1, residuals=tablefit2$V4, observed=tablefit2$V2, predicted=tablefit2$V3))
    }
  }
  outputs_VPA$residuals <- fit_tot
  
  for (k in 1:nRet){
    run2[[k]]@mat[4,] <- 0.5
    file_waa          <- runParameters$waa[which(as.character(runParameters$Run)==name_run)]
    WAA               <- read.table(paste(dirData, file_waa, sep=""), header=F, sep=";")
    colnames(WAA)     <- c("Year", paste("Age",1:age_plus))
    WAAVPA            <- t(WAA[,2:(age_plus+1)])
    
    stock.N           <- stock.n(run2[[k]])[drop=T]
    faa               <- harvest(run2[[k]])[drop=T]
    rownames(faa)     <- c(paste("Age ", 1:(age_plus-1), sep=""), paste("Age ",age_plus,"plus", sep=""))
    stock.N           <- stock.N * exp(-(faa+m)*0.5)
    WAAVPA            <- WAAVPA[,1:dim(stock.N)[2]]
    stock.weight      <- stock.N*WAAVPA 
    
    SSB               <- colSums(stock.weight*mat(run2[[k]])[drop=T])
    recruitment       <- rec(run2[[k]])[drop=T]
    recruitment[(length(recruitment)-2):length(recruitment)]  <- NA
    yield             <- catch(run2[[k]])[drop=T][drop=T]
    fishing_10        <- harvest(run2[[k]])[10][drop=T]
    N10               <- stock.n(run2[[k]])[10][drop=T]
    caa               <- catch.n(run2[[k]])[drop=T]
    fishing_2_5       <- apply(harvest(run2[[k]])[2:5][drop=T],2,function(x) exp(mean(log(x))))
    
    plts=data.frame("Retros"  = k-1,
                    "Year"    = as.numeric(colnames(stock.N)),
                    "SSB"     = SSB,
                    "Recruits"= recruitment,
                    "F2-5"    = fishing_2_5,
                    "Fplusgroup" = fishing_10
    )
    if (k==1) {faa_first <- faa}
    plts1          <- melt(plts, measure.vars=colnames(plts[3:6]))
    plts_tot <- rbind(plts_tot,plts1)
  }
  #plts_tot$variable <- factor(plts_tot$variable, levels=c("F2_5", "F10", "Recruits", "SSB"))
  outputs_VPA$main  <- plts_tot
  outputs_VPA$faa   <- faa_first
  l_dir             <- length(unlist(strsplit(dir_in, split="/")))
  nb_seed           <- as.numeric(unlist(strsplit(dir_in, split="/"))[l_dir])
  save(outputs_VPA, file=paste(dirRData, "data_retro_", name_run, "_", nb_seed,".Rdata", sep="" ))
  ##save as an ncdf:
  OutputsVPA2NetCDF(dirOut_nc =dirNCData,run_name = name_run,nbr_seed = nb_seed,outputs_VPA=outputs_VPA)

  
  # return(outputs_VPA)
}


#################################################################
### Basic function to read the VPA output of a retrospective  ###
### analysis                                                  ###
#################################################################
get_proj_data <- function(dir_in=dir_proj, scen= c("low", "med", "high"), proxy="f0.1"){
  library(kobe)
  dir_proj1 <- paste(dir_in, scen, sep="")
  reported_output_proj <- NULL
  for (i in 1:length(dir_proj1)){
    tempproj             <- readKobe2box(dir=dir_proj1[i], proxy=proxy)
    reported_output_proj <- rbind(reported_output_proj, cbind("Scenario"=scen[i],tempproj))
  }
  return(reported_output_proj)
}

