##################################
### Define the f_ratios        ###
##################################
f_ratio_list_1                                         <- NULL
### F-ratio as used in 2012
f_ratio_list_1$scen1$periods_duration                  <- c(20,15,10,13,6)
f_ratio_list_1$scen1$min_bound_terminal_F_vector_final <- rep(0.1,5)
f_ratio_list_1$scen1$terminal_F_vector_final           <- c(0.7,1,0.6,1.2,1.0)
f_ratio_list_1$scen1$max_bound_terminal_F_vector_final <- rep(0.5,5)
f_ratio_list_1$scen1$indicator_final                   <- rep(0,5)
f_ratio_list_1$scen1$standard_deviation_prior_final    <- rep(0.02,5)
f_ratio_list_1$scen1$reference_age_final               <- rep(10,5)

### F-ratio constant over the period and equal to 1
f_ratio_list_1$scen2$periods_duration                  <- 64
f_ratio_list_1$scen2$min_bound_terminal_F_vector_final <- 0.1
f_ratio_list_1$scen2$terminal_F_vector_final           <- 1.0
f_ratio_list_1$scen2$max_bound_terminal_F_vector_final <- 0.5
f_ratio_list_1$scen2$indicator_final                   <- 0
f_ratio_list_1$scen2$standard_deviation_prior_final    <- 0.02
f_ratio_list_1$scen2$reference_age_final               <- 10

### F-ratio as used in 2012
f_ratio_list_1$scen3$periods_duration                  <- c(1,19,1,14,1,9,1,12,1,5)
f_ratio_list_1$scen3$min_bound_terminal_F_vector_final <- rep(0.1,10)
f_ratio_list_1$scen3$terminal_F_vector_final           <- c(0.7,0.7,1.0,1,0.6,0.6,1.2,1.2,1.0,1.0)
f_ratio_list_1$scen3$max_bound_terminal_F_vector_final <- rep(0.5,10)
f_ratio_list_1$scen3$indicator_final                   <- c(1.0,-0.1,1.0,-0.1,1.0,-0.1,1.0,-0.1,1.0,-0.1)
f_ratio_list_1$scen3$standard_deviation_prior_final    <- rep(0.02,10)
f_ratio_list_1$scen3$reference_age_final               <- rep(10,10)

### F-ratio as Laurie's estimates
f_ratio_list_1$scen4$periods_duration                  <- c(1,9,1,9,1,11,1,10,1,20)
f_ratio_list_1$scen4$min_bound_terminal_F_vector_final <- rep(0.1,10)
f_ratio_list_1$scen4$terminal_F_vector_final           <- c(0.7,0.7,1.0,1,0.6,0.6,1.2,1.2,1.0,1.0)
f_ratio_list_1$scen4$max_bound_terminal_F_vector_final <- rep(0.5,10)
f_ratio_list_1$scen4$indicator_final                   <- c(1.0,-0.1,1.0,-0.1,1.0,-0.1,1.0,-0.1,1.0,-0.1)
f_ratio_list_1$scen4$standard_deviation_prior_final    <- rep(0.02,10)
f_ratio_list_1$scen4$reference_age_final               <- rep(10,10)

############################################
### Generate the different seed numbers  ###
### if nb_seed is selected               ###
############################################
seed_number_gen <- function(nb_seed){
  seed_number <- c(-911,round(runif((nb_seed-1),-999,-1)))
  while(length(unique(seed_number))<nb_seed){seed_number <- c(seed_number,round(runif((nb_seed-length(unique(seed_number))), -999,-1)))}
  seed_number <- seed_number[1:nb_seed]
  save(seed_number, file=paste(dirRData,"/",gsub(" ", "", format(Sys.time(), "%Y %M %d %X")),"_",nb_seed,".Rdata", sep=""))
  return(seed_number)
}

########################################
### Generate the random seed number  ###
########################################
if (exists("nb_seed")==T & exists("r_seed")==T & exists("seed_numb")==T){stop("!!!Choose one way to generate the random seed number but not the three of them!!!")}
if (exists("nb_seed")==F & exists("r_seed")==T & exists("seed_numb")==T){stop("!!!Choose one way to generate the random seed number but not r_seed and seed_numb !!!")}
if (exists("nb_seed")==T & exists("r_seed")==F & exists("seed_numb")==T){stop("!!!Choose one way to generate the random seed number but not nb_seed and r_seed!!!")}
if (exists("nb_seed")==T & exists("r_seed")==T & exists("seed_numb")==F){stop("!!!Choose one way to generate the random seed number but not nb_seed and seed_numb!!!")}

if (exists("nb_seed")) {seed_numb <- seed_number_gen(nb_seed) }
if (exists("ran_seed")){load(file=paste(dirRData, ran_seed,".Rdata", sep=""));
                        tmp       <- load(file= paste(dirRData,"/", ran_seed,".Rdata", sep=""));
                        seed_numb <- eval(parse(text=tmp));
                        rm(list=c(tmp))}
if (sum(seed_numb>0) > 0){stop("Random seed numbers have to be negative values")}
##########################################################
### Read  the run parameters for the selected run(s)   ###
##########################################################
if (sum(runNumb=="all")==1){runNumb <- runParameters$Run}

if (sum(runParameters$Run%in%runNumb)==0){stop(paste('!!! Create your run in ',dirData,'run_spec.csv !!!',sep=""))}
runs  <- runParameters[runParameters$Run%in%runNumb,]

##########################################################
### Create the simulations matrix to be run            ###
##########################################################
if (nbBoot[1]>0 & nbRetro[1]>0){stop("NOT POSSIBLE TO MAKE BOOTSTRAPS AND RETROS AT THE SAME TIME")}
simulations <- expand.grid("nbBoot"       = nbBoot[1],
                           "nbRetros"     = nbRetro[1],
                           "r_seed"       = seed_numb,
#                           "scen_f_ratio" = scen_f_ratio,
                           "Run"          = runs$Run
                            )

if (length(nbBoot)>1 & length(nbRetro)>1){
  if (nbBoot[2]>0 & nbRetro[2]>0){stop("NOT POSSIBLE TO MAKE BOOTSTRAPS AND RETROS AT THE SAME TIME")}
  simulations <- rbind(simulations,expand.grid("nbBoot"       = nbBoot[2],
                                               "nbRetros"     = nbRetro[2],
                                               "r_seed"       = seed_numb,
#                                               "scen_f_ratio" = scen_f_ratio,
                                               "Run"          = runs$Run
                                                ))
}


for (i in 1:dim(simulations)[1]){
  f_ratio_list <- eval(parse(text=paste("f_ratio_list_1$scen",runs$f_ratio[runs$Run==simulations$Run[i]], sep="")))
  ##figure out whether the catch data are inflated or reported
  if (length(grep(pattern = runs$CAA[runs$Run==simulations$Run[i]], x="CAA_inflated.csv"))==1) {type_rep="Inflated"} else {type_rep = "Reported"}
  ##define if it is bootstraps or retrospective analyses
  if (simulations$nbBoot[i]   > 0) {type_analysis <- "Boot"}
  if (simulations$nbRetros[i] > 0) {type_analysis <- "Retros"}
  ## define the path name accordingly and create it if it does not exists
  dirRun      <- paste(dirOutputs, type_rep, "/", simulations$Run[i], "/", type_analysis, "/", abs(simulations$r_seed[i]), sep="")
  if (file.exists(dirRun)==T) {setwd(dirRun)} else {dir.create(dirRun, recursive=T);
                                                      setwd(dirRun); 
                                                      print(paste(dirRun, " folder has been created!", sep=""))}
  cpue_number_1                  <- runs$nCPUE[runs$Run==simulations$Run[i]]
  years_1                        <- c(1950, as.numeric(substr(runs$Year[runs$Run==simulations$Run[i]], 6,9)))
  if (simulations$Run[i]=="Run_6"){cpue_spec <- read.csv(paste(dirData, "cpue_spec_aerial.csv", sep=""))} 
  else {
    cpue_spec <- read.csv(paste(dirData, "cpue_spec.csv", sep=""))}
  cpue_to_use_1                  <- 1:cpue_number_1
  year_max_cpue_1                <- rep(years_1[2], (cpue_number_1[1]))
  cut_all <- 0
  runs$CPUE_cut[runs$Run==simulations$Run[i]]      <- eval(parse(text=as.character(runs$CPUE_cut[runs$Run==simulations$Run[i]])))
  runs$n_remove_year[runs$Run==simulations$Run[i]] <- eval(parse(text=as.character(runs$n_remove_year[runs$Run==simulations$Run[i]])))
  if (runs$CPUE_cut[runs$Run==simulations$Run[i]]!=0){
    for (k in 1:length(runs$CPUE_cut[runs$Run==simulations$Run[i]])){
      year_max_cpue_1[runs$CPUE_cut[runs$Run==simulations$Run[i]][k]] <- years_1[2]-runs$n_remove_year[runs$Run==simulations$Run[i]][k]
      if (runs$n_remove_year[runs$Run==simulations$Run[i]][k]>60) {cut_all=c(cut_all,k)} else (cut_all <- 0)
    }
  }
  if (sum(cut_all>0)>0){
    index_to_remove <- runs$CPUE_cut[runs$Run==simulations$Run[i]][cut_all[cut_all>0]]
    cpue_to_use_1   <- cpue_to_use_1[-index_to_remove]
    cut_all         <- 0
  }

############################################
### Define the filenames where the data  ###
### are for different input of VPA       ###
############################################
file_caa  <- paste(dirData, runParameters$CAA[runParameters$Run==simulations$Run[i]], sep="")
file_cpue <- paste(dirData, runParameters$CPUE[runParameters$Run==simulations$Run[i]], sep="")
file_pcaa <- paste(dirData, runParameters$PCAA[runParameters$Run==simulations$Run[i]], sep="")
file_waa  <- paste(dirData, runParameters$waa[runParameters$Run==simulations$Run[i]], sep="")
id_new    <- length(grep(x=simulations$Run[i], pattern = "new"))

#######################
### READ Rscripts   ###
#######################
### To generate VPA2-BOX files
source(paste(dirRScripts,"writeFuncs.R", sep=""))
### To calculate the weight-at-age with different plus (or minus :)) group
source(paste(dirRScripts,"age_plus_conversion.R", sep=""))

######################################
### SOURCE code to generate files  ###
######################################

write_d1VPA(dir_out       = dirRun,
            years         = years_1,
            cpue_number   = cpue_number_1,
            cpue_to_use   = cpue_to_use_1,
            year_max_cpue = year_max_cpue_1,
            cpue_spec     = cpue_spec,
            age_minus     = 0,
            age_plus      = runParameters$age_plus_group[runParameters$Run==simulations$Run[i]],
            dir_data      = dirData)
### write control file 
write_c1VPA(dir_out                            = dirRun,
            years                              = years_1,
            age_minus                          = 0,
            age_plus                           = runParameters$age_plus_group[runParameters$Run==simulations$Run[i]],
            random_seed                        = simulations$r_seed[i],
            number_bootstraps                  = simulations$nbBoot[i],
            number_year_retrospective_analyses = simulations$nbRetros[i])
### write parameter file
write_p1VPA(dir_out       = dirRun,
            age_minus     = 0,
            age_plus      = runParameters$age_plus_group[runParameters$Run==simulations$Run[i]],
            cpue_number   = cpue_number_1)

filename_c1_vpa <- paste(filenameVpa,".c1", sep="")
#####################################
### DETERMINE THE EXECUTABLE FILE ###
#####################################
if (Sys.info()[1]=="Linux")  {vpaExeFile  <- paste(dirVpaFiles, "vpa-2box.out", sep="")
                              file.copy(from=vpaExeFile, to=dirRun, copy.mode = T, overwrite=T)
                              system(paste("./vpa-2box.out", filename_c1_vpa, sep=" "))
                              # file.remove("vpa-2box.out") # I think it worth it to keep the exe to keep track of what has been used
                              }
if (Sys.info()[1]=="Windows"){ vpaExeFile  <- paste(dirVpaFiles, "vpa-2box.exe", sep="")
                               file.copy(from=vpaExeFile, to=dirRun)
                               system(paste("yes | wine vpa-2box.exe", " " , filename_c1_vpa, sep=""))
                              # file.remove("vpa-2box.exe") # I think it worth it to keep the exe to keep track of what has been used
                              }

}



  



