#######################
### LOAD libraries  ###
#######################
require(plyr)
library(reshape)
library(ggplot2)
library(RColorBrewer)
library(lattice)
library(xtable)
library(knitr)
library(FLa4a)
library(kobe)
library(shiny)

### REMOVE evrything to be sure
rm(list=ls())

##########################
### DEFINE YOUR PATH  ####
##########################
dirMain     <- "/media/sbonhomm/Caparmor/syl/boulot/Projet/En_cours/2015_emarine/julien/"

######################################################################
### The rest of the scripts uses relative path to the dirMain one ####
######################################################################
dirRScripts <- paste(dirMain, "Rscripts/", sep="")
dirVpaFiles <- paste(dirMain, "vpafiles/", sep="")
dirData     <- paste(dirMain, "data/", sep="")
dirRData    <- paste(dirMain, "Rdata/", sep="")
dirOutputs  <- paste(dirMain, "outputs/", sep="")

##############################################################################################################
### STEP 1: RUN THE VPA FROM SPECIFICATIONS                                                                ###
##############################################################################################################

#########################################
### DEFINE YOUR RUN'S SPECIFICATIONs  ###
#########################################
### Parameters of the runs, names and cpues
### !!! Modify the run_spec.csv file if you want to add a Run !!!
### Basically, this is a file to change the input data and the parameter values for different sensitivity analysis
runParameters <- read.csv(paste(dirData,'run_spec.csv',sep=""))

#############################################
#### Select the run to run                ###
#############################################
#############################################
#### Select the run number you want       ###
#### (from the "run_spec.csv" file)       ###
#### to run them all, type "all"          ###
#############################################
#Examples:
#runNumb     <- "Run_13"
#runNumb     <- c("Run_0_A","Run_7")
#runNumb     <- "all"
#runNumb     <- as.character(runParameters$Run[52:73])
runNumb     <- "Run_5"

#############################################
#### Define file names for the vpa        ###
#############################################
filenameVpa  <- "bfte2014"

#############################################################################################
### Select the seed number to use for the run   (should be a negative integer)            ###
### or specify a number of random seed you want to get (positive integer)                 ###
### or use the random seeds used previously, give the Rdata filename   "r_seed_2014"      ###
#############################################################################################
#Examples:
#seed_numb  <- -543
#nb_seed    <- 100
#ran_seed   <- "r_seed_2014"
#ran_seed <- "2015530204:53:47_4"
seed_numb   <- -911

########################################################
### Select the number of retrosspective analyses     ###
### to be done                                       ###
### !! you cannot make bootstraps and retros at the  ###
### time (the code will stop)                        ###
########################################################
#Examples: 
#nbBoot      <- 0
#nbRetro     <- 0
###nb_boot <- c(0,1000)
###nb_retro <- c(6,0)
nbRetro     <- 5
nbBoot      <- 0

###########################################################
### Define the age plus group and the age minus group   ###
### or a sequence of age you want to test               ###
# ###########################################################
# age_plus_1   <- 10
# age_minus_1  <- 0

###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
###!!! F_ratio not needed anymore but specified in the run_spec.csv file
###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
####################################################
### Select the F-ratio scenario you want to run  ###
### 4 scenarios available see run_generator.R    ###
### for their description and add another one    ###
### if you want                                  ###
### 1: is as in 2012                             ###
### 2: is same periods as in 2012 but estimated  ###
### 3: is periods estimated using a catch curve  ###
### 4: is F-ratio=1 for the whole period         ###
####################################################
#Examples:
#scen_f_ratio <- 1
#scen_f_ratio <- c(1,2,4)
#scen_f_ratio <- c(1,2,3,4)
###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

## Just to benchmark the time to run the calculations for this step
timeTestStart <- Sys.time()
source(paste(dirRScripts, "run_generator.R", sep=""))
timeTestEnd <- Sys.time()
timeTestEnd-timeTestStart

##############################################################################################################
### STEP 2: GET THE VPA OUTPUTS                                                                            ###
##############################################################################################################

#######################################################
### Define from which run you want to get the data  ###
### either one specific, different runs, or "all"   ###
#######################################################
#get_all_retros <- "all"  ### to get all retros
get_all_retros <- "some"
#get_all_proj   <- "all"  ### to get all projections
#run_to_get <- c("Run_13")
#run_to_get         <- paste("Run_5_out",1:7, sep="") # c("Run_3", "Run_4", "Run_7", paste("Run_",14:17, sep=""), "Run_5_new", paste("Run_", 14:16, "_new", sep="")) #"Run_5_inflate"#paste("Run_5_",1:6, sep="")#inflate")# c("Run_11_new", "Run_12_new", "Run_13_new")#,"Run_4", "Run_5", "Run_7", "Run_5_new")# , "Run_5_inflate"#c(paste("Run_5_out",1:7,sep="")) #c(paste("Run_",3:7,"_new",sep=""),"Run_11_new", "Run_12_new", "Run_13_new")
###seed_number_to_get <- c(-911,-104)
#seed_number_to_get <- 911  
seed_number_to_get <- 911
###what_data <- c("Retros","Projections")
what_data <- "Retros"
catch_type<- c("Reported", "Inflated")
#catch_type <- "Reported"

source(paste(dirRScripts,"get_runs_functions.R", sep=""))
source(paste(dirRScripts,"get_runs_code.R", sep=""))

##########################################################
### Select the run that seem appropriate for the group ###
### For this purpose, a Rshiny app has been developped ### 
##########################################################
### To view all the runs with the all the seed numbers
runApp(paste(dirRScripts,"shiny_app1", sep=""))

### To view all the runs with only the best objective function or AIC or AICc or BIC
source(paste(dirRScripts, "dir_best.R", sep=""))
runApp(paste(dirRScripts,"shiny_app2", sep=""))

##############################################################################################################
### STEP 3: RUN THE PROJECTIONS ON THE SEKECTED MODEL                                                      ###
##############################################################################################################

##########################################################
#### Bootstraps the selected run(s)                   ###
#### (from the "run_spec.csv" file)                    ###
#### to run them all, type "all"                       ###
##########################################################
### Selected run(s)
#Examples:
#runNumb     <- "Run_13"
#runNumb     <- c("Run_5","Run_7")
#runNumb     <- "all"
filenameVpa  <- "bfte2014"
runNumb     <- "Run_5"
seed_numb   <- -911
nbBoot      <- 1000
nbRetro     <- 0

timeTestStart <- Sys.time()
source(paste(dirRScripts, "run_generator.R", sep=""))
timeTestEnd <- Sys.time()
timeTestEnd-timeTestStart

# Projections
run_name1 <- "Run_5"
seed_numb <- -911
current_quota_1 <- 13500

source(paste(dirRScripts, "writeFuncs_proj.R", sep=""))
source(paste(dirRScripts, "proj_creator.R", sep=""))

# SHiny app of the results

# Automatic report

##########################################################


